@@ -4,9 +4,11 @@ import traceback
 from dataclasses import dataclass
 from functools import partial
 from multiprocessing import Event, Manager, Process, Value, Queue
+
 from typing import Dict, Optional
 
 from skyplane.chunk import ChunkRequest
+from skyplane.config_paths import cloud_config
 from skyplane.gateway.chunk_store import ChunkStore
 from skyplane.obj_store.object_store_interface import ObjectStoreInterface
 from skyplane.utils import logger
@@ -26,6 +28,7 @@ class GatewayObjStoreConn:
         self.error_queue = error_queue
         self.n_processes = max_conn
         self.processes = []
+        self.src_requester_pays = cloud_config.get_flag("requester_pays")
 
         # shared state
         self.manager = Manager()
@@ -44,6 +47,7 @@ class GatewayObjStoreConn:
         if key not in self.obj_store_interfaces:
             logger.warning(f"[gateway_daemon] ObjectStoreInterface not cached for {key}")
             try:
+                logger.debug(f"[gateway_daemon] Creating ObjectStoreInterface for {key}")
                 self.obj_store_interfaces[key] = ObjectStoreInterface.create(region, bucket)
             except Exception as e:
                 raise ValueError(f"Failed to create obj store interface {str(e)}")
@@ -98,13 +102,14 @@ class GatewayObjStoreConn:
                             chunk_req.chunk.part_number,
                             chunk_req.chunk.upload_id,
                             check_md5=chunk_req.chunk.md5_hash,
+                            mime_type=chunk_req.chunk.mime_type,
                         ),
                         max_retries=4,
                     )
                     chunk_file_path = self.chunk_store.get_chunk_file_path(chunk_req.chunk.chunk_id)
                     self.chunk_store.state_finish_upload(chunk_req.chunk.chunk_id, f"obj_store:{self.worker_id}")
                     chunk_file_path.unlink()
-                    logger.debug(f"[obj_store:{self.worker_id}] Uploaded {chunk_req.chunk.chunk_id} to {bucket}")
+                    logger.info(f"[obj_store:{self.worker_id}] Uploaded {chunk_req.chunk.chunk_id} to {bucket}")
                 elif req_type == "download":
                     assert chunk_req.src_type == "object_store"
 
@@ -117,7 +122,10 @@ class GatewayObjStoreConn:
                     logger.debug(f"[obj_store:{self.worker_id}] Start download {chunk_req.chunk.chunk_id} from {bucket}")
 
                     obj_store_interface = self.get_obj_store_interface(chunk_req.src_region, bucket)
-                    md5sum = retry_backoff(
+
+                    if self.src_requester_pays:
+                        obj_store_interface.set_requester_bool(True)
+                    mime_type, md5sum = retry_backoff(
                         partial(
                             obj_store_interface.download_object,
                             chunk_req.chunk.src_key,
@@ -134,13 +142,19 @@ class GatewayObjStoreConn:
                         logger.error(f"[obj_store:{self.worker_id}] Checksum was not generated for {chunk_req.chunk.src_key}")
                     else:
                         self.chunk_store.update_chunk_checksum(chunk_req.chunk.chunk_id, md5sum)
+                    if not mime_type:
+                        logger.error(f"[obj_store:{self.worker_id}] Mime type was not generated for {chunk_req.chunk.src_key}")
+                    else:
+                        self.chunk_store.update_chunk_mime_type(chunk_req.chunk.chunk_id, mime_type)
 
                     self.chunk_store.state_finish_download(chunk_req.chunk.chunk_id, f"obj_store:{self.worker_id}")
                     recieved_chunk_size = self.chunk_store.get_chunk_file_path(chunk_req.chunk.chunk_id).stat().st_size
                     assert (
                         recieved_chunk_size == chunk_req.chunk.chunk_length_bytes
                     ), f"Downloaded chunk {chunk_req.chunk.chunk_id} to {fpath} has incorrect size (expected {chunk_req.chunk.chunk_length_bytes} but got {recieved_chunk_size}, {chunk_req.chunk.chunk_length_bytes})"
-                    logger.debug(f"[obj_store:{self.worker_id}] Downloaded {chunk_req.chunk.chunk_id} from {bucket}")
+                    logger.debug(
+                        f"[obj_store:{self.worker_id}] Downloaded {chunk_req.chunk.chunk_id} from {bucket} with {md5sum} and {mime_type}"
+                    )
                 else:
                     raise ValueError(f"Invalid location for chunk req, {req_type}: {chunk_req.src_type}->{chunk_req.dst_type}")
             except Exception as e:
